# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


class MattermostNotifyError(Exception):
    """Base error for all errors originating from mattermost-notify"""
